--[[
For quick-cocos2d-x
SocketTCP lua
@author zrong (zengrong.net)
Creation: 2013-11-12
Last Modification: 2013-12-05
@see http://cn.quick-x.com/?topic=quickkydsocketfzl
]]
local SOCKET_TICK_TIME = 0.1            -- check socket data interval
local SOCKET_RECONNECT_TIME = 5         -- socket reconnect try interval
local SOCKET_CONNECT_FAIL_TIMEOUT = 3   -- socket failure timeout

local STATUS_CLOSED = "closed"
local STATUS_NOT_CONNECTED = "Socket is not connected"
local STATUS_ALREADY_CONNECTED = "already connected"
local STATUS_ALREADY_IN_PROGRESS = "Operation already in progress"
local STATUS_TIMEOUT = "timeout"

local scheduler = require("framework.scheduler")
local socket = require "socket"

local byteArray = require("src.framework.cc.utils.ByteArray")

local J20SocketTCP = class("J20SocketTCP")

J20SocketTCP.EVENT_DATA = "SOCKET_TCP_DATA"
J20SocketTCP.EVENT_CLOSE = "SOCKET_TCP_CLOSE"
J20SocketTCP.EVENT_CLOSED = "SOCKET_TCP_CLOSED"
J20SocketTCP.EVENT_CONNECTED = "SOCKET_TCP_CONNECTED"
J20SocketTCP.EVENT_CONNECT_FAILURE = "SOCKET_TCP_CONNECT_FAILURE"

J20SocketTCP._VERSION = socket._VERSION
J20SocketTCP._DEBUG = socket._DEBUG

function J20SocketTCP.getTime()
    return socket.gettime()
end

function J20SocketTCP:ctor(__host, __port, __retryConnectWhenFailure)
    cc(self):addComponent("components.behavior.EventProtocol"):exportMethods()

    self.host = __host
    self.port = __port
    self.tickScheduler = nil            -- timer for data
    self.reconnectScheduler = nil       -- timer for reconnect
    self.connectTimeTickScheduler = nil -- timer for connect timeout
    self.name = 'J20SocketTCP'
    self.tcp = nil
    self.isRetryConnect = __retryConnectWhenFailure
    self.isConnected = false
end

function J20SocketTCP:setName( __name )
    self.name = __name
    return self
end

function J20SocketTCP:setTickTime(__time)
    SOCKET_TICK_TIME = __time
    return self
end

function J20SocketTCP:setReconnTime(__time)
    SOCKET_RECONNECT_TIME = __time
    return self
end

function J20SocketTCP:setConnFailTime(__time)
    SOCKET_CONNECT_FAIL_TIMEOUT = __time
    return self
end

function J20SocketTCP:connect(__host, __port, __retryConnectWhenFailure)
    if __host then self.host = __host end
    if __port then self.port = __port end
    if __retryConnectWhenFailure ~= nil then self.isRetryConnect = __retryConnectWhenFailure end
    assert(self.host or self.port, "Host and port are necessary!")
    --printInfo("%s.connect(%s, %d)", self.name, self.host, self.port)
    local isipv6_only = false
    local addrifo,err = socket.dns.getaddrinfo(self.host)
    if addrifo ~= nil then
        for k,v in pairs(addrifo) do
            if v.family == "inet6" then
                isipv6_only = true
                break
            end
        end
    end
    if isipv6_only then
        self.tcp = socket.tcp6()
    else
        self.tcp = socket.tcp()
    end
    
    self.tcp:settimeout(0)

    local function __checkConnect()
        local __succ = self:_connect()
        if __succ then
            self:_onConnected()
        end
        return __succ
    end

    if not __checkConnect() then
        -- check whether connection is success
        -- the connection is failure if socket isn't connected after SOCKET_CONNECT_FAIL_TIMEOUT seconds
        local __connectTimeTick = function ()
            --printInfo("%s.connectTimeTick", self.name)
            if self.isConnected then return end
            self.waitConnect = self.waitConnect or 0
            self.waitConnect = self.waitConnect + SOCKET_TICK_TIME
            if self.waitConnect >= SOCKET_CONNECT_FAIL_TIMEOUT then
                self.waitConnect = nil
                self:_connectFailure()
                self:close()
            end
            __checkConnect()
        end
        self.connectTimeTickScheduler = scheduler.scheduleGlobal(__connectTimeTick, SOCKET_TICK_TIME)
    end
end

function J20SocketTCP:send(__data)
    assert(self.isConnected, self.name .. " is not connected.")
    self.tcp:send(__data)
end

function J20SocketTCP:close( ... )
    --printInfo("%s.close", self.name)
    self.tcp:close();
    if self.connectTimeTickScheduler then scheduler.unscheduleGlobal(self.connectTimeTickScheduler) end
    if self.tickScheduler then scheduler.unscheduleGlobal(self.tickScheduler) end
    self:dispatchEvent({name=J20SocketTCP.EVENT_CLOSE})
end

-- disconnect on user's own initiative.
function J20SocketTCP:disconnect()
    self:_disconnect()
    self.isRetryConnect = false -- initiative to disconnect, no reconnect.
end

--------------------
-- private
--------------------

--- When connect a connected socket server, it will return "already connected"
-- @see: http://lua-users.org/lists/lua-l/2009-10/msg00584.html
function J20SocketTCP:_connect()
    local __succ, __status = self.tcp:connect(self.host, self.port)
    -- print("SocketTCP._connect:", __succ, __status)
    return __succ == 1 or __status == STATUS_ALREADY_CONNECTED
end

function J20SocketTCP:_disconnect()
    self.isConnected = false
    self.tcp:shutdown()
    self:dispatchEvent({name=J20SocketTCP.EVENT_CLOSED})
end

function J20SocketTCP:_onDisconnect()
    --printInfo("%s._onDisConnect", self.name);
    self.isConnected = false
    self:dispatchEvent({name=J20SocketTCP.EVENT_CLOSED})
    self:_reconnect();
end


function J20SocketTCP:forceClose()
    self.tcp:close();
    if self.connectTimeTickScheduler then scheduler.unscheduleGlobal(self.connectTimeTickScheduler) end
    if self.tickScheduler then scheduler.unscheduleGlobal(self.tickScheduler) end
end


-- connecte success, cancel the connection timerout timer
function J20SocketTCP:_onConnected()
    --printInfo("%s._onConnectd", self.name)
    self.isConnected = true
    self:dispatchEvent({name=J20SocketTCP.EVENT_CONNECTED})
    if self.connectTimeTickScheduler then scheduler.unscheduleGlobal(self.connectTimeTickScheduler) end

    
    local __tick = function()
        while true do
            local __body, __status, __partial = self.tcp:receive(4)  -- read the package body
            
            
            if __status == STATUS_CLOSED or __status == STATUS_NOT_CONNECTED then
                if self.isConnected then
                    self:_onDisconnect()
                else
                    self:_connectFailure()
                end
                self:close()
                return
            end
            
            
            if __body ~= nil then
                
                local bytes = byteArray:new()
           
                for i=1,4 do
                    bytes:writeRawByte(string.sub(__body,5-i,5-i))
                end
                
                bytes:setPos(1)
               
                local len = bytes:readInt()
                
                local mydata = self.tcp:receive(len)
                
                local t1 = J20NetDataReceive:new(mydata,len)
                
                local msg = t1:getMsg()
                local data = t1:getData()
                local code = t1:getCode()
                
                local callback = t1:getCallBack()
                local iscompress = t1:getIsCompress()
                local cjson = require("cjson")
                local recvdata = cjson.decode(data) 
                
                --FlyWordPrompt:prompt("r:"..callback.." d:"..string.sub(data,1,20) )
                
                  
                if callback and callback ~= ""  then
                    self:dispatchEvent({name=callback,code=code,data=(recvdata)})
                end
                return
                --self:dispatchEvent({name=J20SocketTCP.EVENT_DATA,code=code,data=(mydata),datalen=(len),partial=__partial, body=__body})
            else
                return
            end
            
            
            --__body, __status, __partial = self.tcp:receive(11)
--            if  (__body and string.len(__body) == 0) or
--                (__partial and string.len(__partial) == 0)
--            then return end
--            if __body and __partial then __body = __body .. __partial end
            --self:dispatchEvent({name=SocketTCP.EVENT_DATA, data=(__partial or __body), partial=__partial, body=__body})
        end
    end

    -- start to read TCP data
    self.tickScheduler = scheduler.scheduleGlobal(__tick, SOCKET_TICK_TIME)
end

function J20SocketTCP:_connectFailure(status)
    --printInfo("%s._connectFailure", self.name);
    self:dispatchEvent({name=J20SocketTCP.EVENT_CONNECT_FAILURE})
    self:_reconnect();
end

-- if connection is initiative, do not reconnect
function J20SocketTCP:_reconnect(__immediately)
    if not self.isRetryConnect then return end
    printInfo("%s._reconnect", self.name)
    if __immediately then self:connect() return end
    if self.reconnectScheduler then scheduler.unscheduleGlobal(self.reconnectScheduler) end
    local __doReConnect = function ()
        self:connect()
    end
    self.reconnectScheduler = scheduler.performWithDelayGlobal(__doReConnect, SOCKET_RECONNECT_TIME)
end

return J20SocketTCP
